
/* 
 * burst-analysis.c : 
 *  computes min/mean/max/stddev and probability of loss burst lengths
 *
 */

#include <assert.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

struct burst {
    double time;
    int len;
};

int comp(const void *x, const void*y) {

	struct burst burstx = *(struct burst *)x;
	struct burst bursty = *(struct burst *)y;
	
	if (burstx.len < bursty.len) {
		return -1;
	} else if (burstx.len > bursty.len) {
		return 1;
	} else {
		return 0;
	}
}

int main(int argc, char *argv[]) {

    struct burst 	bursts[atoi(argv[1])];
    int 			num_bursts = 0;
	int 			num_losses = 0;
	int				min_len	   = 1e6;
	double 			mean_len   = 0;
	int				max_len	   = 0;
	double 			median_len = 0;
	double 			stddev_len = 0;
    char 			buf[50];
    
	while (fgets(buf, sizeof(buf), stdin) != NULL) {
        sscanf(buf, "%lg %d", &(bursts[num_bursts].time), &(bursts[num_bursts].len));
		num_losses += bursts[num_bursts].len;
		if (bursts[num_bursts].len < min_len) min_len = bursts[num_bursts].len;
		if (bursts[num_bursts].len > max_len) max_len = bursts[num_bursts].len;
        num_bursts++;
    }
    
	printf("# %4s %11s %12s\n", "len", "num_bursts", "P[burst]");

    /* go through different burst lengths (up to max) */
    for(int n = 1; n <= max_len; n++) {
        int 	num_n_bursts = 0; /* num bursts of length n */
		double 	prob_n; /* probability of burst of length n */

        for(int i = 0; i < num_bursts; i++) {
            /* check for bursts of length *exactly* n */
			if (bursts[i].len == n) {
                num_n_bursts++;
			}
        }

		prob_n = (double)num_n_bursts / (double)num_bursts;	
        if (num_n_bursts > 0) {
            printf("  % 4d % 11d %12g\n", n, num_n_bursts, prob_n);
		}
    }

	/* mean and standard deviation */
	mean_len = (double)num_losses / (double)num_bursts;
	for (int i = 0; i < num_bursts; i++) {
		double diff = (bursts[i].len - mean_len);
		stddev_len += (diff*diff);
	}
	assert(stddev_len >= 0);
	stddev_len = sqrt(stddev_len / num_bursts);

	/* median */
	qsort(bursts, num_bursts, sizeof(struct burst), &comp);
	if ((num_bursts % 2) == 1) {
		median_len = bursts[(int)num_bursts/2].len;
	} else {
		median_len = (bursts[(int)num_bursts/2].len + bursts[((int)num_bursts/2)-1].len)/2.0;
	}

	printf("#\n");
	printf("# num_bursts 		: %d\n", num_bursts);
	printf("# min_burst_len 	: %d\n", min_len);	
	printf("# mean_burst_len 	: %g\n", mean_len);
	printf("# median_burst_len 	: %g\n", median_len);
	printf("# max_burst_len 	: %d\n", max_len);	
	printf("# stddev_burst_len 	: %g\n", stddev_len);
	printf("# coefvar_burst_len	: %g\n", stddev_len/mean_len);

    return 0;
}
