#!/bin/bash

read_delay_vals () {
	basename=$1; shift
	
	lrfile=$basename.loss_reordering
	ctfile=$basename.corrected_transit.gz
	ctblfile=$basename.corrected_transit_before_loss
	ctalfile=$basename.corrected_transit_after_loss
	qdfile=$basename.qdelay
	qdavgfile=$basename.qdelay_avg
	qdblfile=$basename.qdelay_before_loss
	qdalfile=$basename.qdelay_after_loss

	# if no ctfile, exit
	if [ ! -s $ctfile ]
	then
		return 1
	fi		

	min_transit=`gunzip -c $ctfile | awk -v basename=$basename '
		BEGIN {
			n = 0;
			min = 999999;
			outf_qdavg = basename ".qdelay_avg"
			outf_ctmin = basename ".corrected_transit_min"
			outf_qd    = basename ".qdelay"
		}
		{
			times[n] = $1;
			transits[n] = $2;
			if (($2 > 0) && ($2 < min)) min = $2;
			n++;
		}
		END {
			qd_sum = 0;
			for (i = 0; i < n; i++) {
				qdelays[i] = transits[i] - min;
				qd_sum += qdelays[i];
				print times[i], qdelays[i] > outf_qd
			}
			print (qd_sum / n) > outf_qdavg
			print min          > outf_ctmin
			print min
		}
	'`
	avg_qdelay=`cat $qdavgfile`
	num_congestive=0
	num_noncongestive=0
	
	if [ -e $ctblfile ]
	then
		cat $ctblfile | awk -v mintransit=$min_transit -v avgqd=$avg_qdelay -v basename=$basename '
			BEGIN {
				congestive = 0;
				noncongestive = 0;
				n = 0;
				outf_c = basename ".loss_congestion"
			}
			{
				qd = ($3 - mintransit)
				print $1, $2, qd
				if (qd > 2*avgqd) {
					congestive++;
				} else {
					noncongestive++;
				}
				n++;
			}
			END {
				printf "congestive : %d : %f %%\nnoncongestive : %d : %f %%", congestive, congestive/n*100, noncongestive, noncongestive/n*100 > outf_c
			}
		' > $qdblfile
	fi

	if [ -e $ctalfile ]
	then
		cat $ctalfile | awk -v mintransit=$min_transit '
			{
				print $1, $2, ($3 - mintransit)
			}
		' > $qdalfile
	fi
}

